%% This code estimates the Taylor and fiscal rules with M-S coefficients
% jointly to obtain the probability of a hawkish monetary policy regime
% IN THIS ESTIMATION, THE SWITCHING OF THE VARIANCE OF THE INTEREST RATE
% EQUATION IS ASSUMED TO BE CORRELATED WITH THE OTHER SWITCHING STATES, 
% EXCEPT THE SWITCHING STATE OF TAXES.
%
% The policy rules are specified as
%
% Rstar_t = rho_R(S_{mt})*Rstar_{t-1} + (1-rho_R(S_{mt}))*[Rstar(S_{mt}) + alpha^p(S_{mt})*p_t + alpha^y(S_{mt})*y_t] + sigma(S_{sRt})e_Rt, e_Rt ~ N(0,1)
% R_t = max(Rstar_t,0.25)
%
% tau_t = rho_tau(S_{ft})*tau_{t-1} + (1-rho_tau(S_{ft}))*[tau(S_{ft}) + gamma^b(S_{ft})*b_{t-1} + gamma^y(S_{ft})*y_t] + sigma(S_{staut})e_taut, e_taut ~ N(0,1)
%
% where
%
% S_{mt} = 1,2,...,J_m
% S_{ft} = 1,2,...,J_f
% S_{sRt} = 1,2,...,J_sR
% S_{staut} = 1,2,...,J_stau
% J_m = 2
% J_f = 2
% J_sR = 2
% J_stau = 2
% 
% S_t = S_{mt} + (S_{ft}-1)J_f = 1,2,...,J
% J = J_m + (J_f - 1)J_f = 4
%
% P(S_t = j|S_{t-1} = j') = p_{jj'}
% P(S_{sRt} = jsR|S_{sRt-1} = jsR') = p_{jsRjsR'}
% P(S_{staut} = jstau|S_{staut-1} = jstau') = p_{jstaujstau'}
%
% It is assumed that p_t and y_t are endogenous and an endogeneity
% correction has to be made.
% 
% 
% Manuel Gonzalez-Astudillo
% 1/9/2017
%% Loading Data
clc,clear;
% data = [R_t tau_t p_t y_t b_{t-1} v_p v_y]
data = load('data_new_pc_net.txt');
vc_upuy = load('vc_upuy_new_pc_net.txt');
Sigmasqrt = chol(vc_upuy); % upper triangular
v = data(:,6:7)/Sigmasqrt; % Standardized Residuals
data(:,6:7) = v;

PSmf = [0.6 0.05 0.05 0.05;
        0.05 0.05 0.05 0.05;
        0.05 0.05 0.05 0.05;
        0.3 0.85 0.85 0.85];
PSsR = [0.9 0.1;
        0.1 0.9];
PSstau = [0.9 0.1;
          0.1 0.9];

%% Setting Maximum Likelihood Estimation
param0 = [log(3.4); log(2);  log(0.5); log(1.9); log(1.7); -1.1; -0.3; 0.3; 0.6; -0.55; log(8.7); log(9.2); log(0.25); log(0.8);...
          log(2.8); log(1.5); 0.02; log(0.13); log(0.4); -0.3; 0.02; 0.26; log(7.5); log(9); log(0.08); log(0.35);...
          transformedprobabilities(kron(PSsR,PSmf));...
          transformedprobabilities(PSstau)];      
options = optimset('Diagnostics','on','Display','iter','FunValCheck','off','MaxFunEvals',100000,'MaxIter',10000);
[param_depvarR,fval,exitflag,output,grad,hessian] = fminunc(@(param)loglikfun_depvarR(param,data),param0,options);
save('param_depvarR','param_depvarR');

%% Obtaining Results
paramm = param_depvarR(1:26);
R_1 = exp(paramm(1));
R_2 = exp(paramm(2));
alphap_1 = exp(paramm(3));
alphap_2 = alphap_1 + exp(paramm(4));
alphay_1 = exp(paramm(5));
alphay_2 = alphay_1 + paramm(6);
kappaRp_1 = paramm(7);
kappaRp_2 = kappaRp_1 + paramm(8);
kappaRy_1 = paramm(9);
kappaRy_2 = kappaRy_1 + paramm(10);
rhoR_1 = exp(paramm(11))/(1+exp(paramm(11)));
rhoR_2 = exp(paramm(12))/(1+exp(paramm(12)));
sigmaR_1 = exp(paramm(13));
sigmaR_2 = sigmaR_1 + exp(paramm(14));
tau_1 = exp(paramm(15));
tau_2 = exp(paramm(16));
gammab_1 = paramm(17);
gammab_2 = gammab_1 + exp(paramm(18));
gammay_1 = exp(paramm(19));
gammay_2 = gammay_1 + paramm(20);
kappatauy_1 = paramm(21);
kappatauy_2 = kappatauy_1 + paramm(22);
rhotau_1 = exp(paramm(23))/(1+exp(paramm(23)));
rhotau_2 = exp(paramm(24))/(1+exp(paramm(24)));
sigmatau_1 = exp(paramm(25));
sigmatau_2 = sigmatau_1 + exp(paramm(26));

betaR_11 = [rhoR_1 (1-rhoR_1)*[R_1 alphap_1 alphay_1] sigmaR_1*kappaRp_1 sigmaR_1*kappaRy_1]';
betaR_12 = [rhoR_1 (1-rhoR_1)*[R_1 alphap_1 alphay_1] sigmaR_2*kappaRp_1 sigmaR_2*kappaRy_1]';
betaR_21 = [rhoR_2 (1-rhoR_2)*[R_2 alphap_2 alphay_2] sigmaR_1*kappaRp_2 sigmaR_1*kappaRy_2]';
betaR_22 = [rhoR_2 (1-rhoR_2)*[R_2 alphap_2 alphay_2] sigmaR_2*kappaRp_2 sigmaR_2*kappaRy_2]';
betatau_11 = [rhotau_1 (1-rhotau_1)*[tau_1 gammab_1 gammay_1] sigmatau_1*kappatauy_1]';
betatau_12 = [rhotau_1 (1-rhotau_1)*[tau_1 gammab_1 gammay_1] sigmatau_2*kappatauy_1]';
betatau_21 = [rhotau_2 (1-rhotau_2)*[tau_2 gammab_2 gammay_2] sigmatau_1*kappatauy_2]';
betatau_22 = [rhotau_2 (1-rhotau_2)*[tau_2 gammab_2 gammay_2] sigmatau_2*kappatauy_2]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% S
% j = 1 <-- jmf = 1 (jm = 1, jf = 1), jsR = 1, jstau = 1
% j = 2 <-- jmf = 2 (jm = 2, jf = 1), jsR = 1, jstau = 1
% j = 3 <-- jmf = 3 (jm = 1, jf = 2), jsR = 1, jstau = 1
% j = 4 <-- jmf = 4 (jm = 2, jf = 2), jsR = 1, jstau = 1
% j = 5 <-- jmf = 1 (jm = 1, jf = 1), jsR = 2, jstau = 1
% j = 6 <-- jmf = 2 (jm = 2, jf = 1), jsR = 2, jstau = 1
% j = 7 <-- jmf = 3 (jm = 1, jf = 2), jsR = 2, jstau = 1
% j = 8 <-- jmf = 4 (jm = 2, jf = 2), jsR = 2, jstau = 1
% j = 9 <-- jmf = 1 (jm = 1, jf = 1), jsR = 1, jstau = 2
% j = 10 <-- jmf = 2 (jm = 2, jf = 1), jsR = 1, jstau = 2
% j = 11 <-- jmf = 3 (jm = 1, jf = 2), jsR = 1, jstau = 2
% j = 12 <-- jmf = 4 (jm = 2, jf = 2), jsR = 1, jstau = 2
% j = 13 <-- jmf = 1 (jm = 1, jf = 1), jsR = 2, jstau = 2
% j = 14 <-- jmf = 2 (jm = 2, jf = 1), jsR = 2, jstau = 2
% j = 15 <-- jmf = 3 (jm = 1, jf = 2), jsR = 2, jstau = 2
% j = 16 <-- jmf = 4 (jm = 2, jf = 2), jsR = 2, jstau = 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
paramp1 = param_depvarR(27:33);
p_11 = exp(paramp1(1))/(1+exp(paramp1(1))+exp(paramp1(2))+exp(paramp1(3))+exp(paramp1(4))+exp(paramp1(5))+exp(paramp1(6))+exp(paramp1(7)));
p_21 = exp(paramp1(2))/(1+exp(paramp1(1))+exp(paramp1(2))+exp(paramp1(3))+exp(paramp1(4))+exp(paramp1(5))+exp(paramp1(6))+exp(paramp1(7)));
p_31 = exp(paramp1(3))/(1+exp(paramp1(1))+exp(paramp1(2))+exp(paramp1(3))+exp(paramp1(4))+exp(paramp1(5))+exp(paramp1(6))+exp(paramp1(7)));
p_41 = exp(paramp1(4))/(1+exp(paramp1(1))+exp(paramp1(2))+exp(paramp1(3))+exp(paramp1(4))+exp(paramp1(5))+exp(paramp1(6))+exp(paramp1(7)));
p_51 = exp(paramp1(5))/(1+exp(paramp1(1))+exp(paramp1(2))+exp(paramp1(3))+exp(paramp1(4))+exp(paramp1(5))+exp(paramp1(6))+exp(paramp1(7)));
p_61 = exp(paramp1(6))/(1+exp(paramp1(1))+exp(paramp1(2))+exp(paramp1(3))+exp(paramp1(4))+exp(paramp1(5))+exp(paramp1(6))+exp(paramp1(7)));
p_71 = exp(paramp1(7))/(1+exp(paramp1(1))+exp(paramp1(2))+exp(paramp1(3))+exp(paramp1(4))+exp(paramp1(5))+exp(paramp1(6))+exp(paramp1(7)));
p_81 = 1/(1+exp(paramp1(1))+exp(paramp1(2))+exp(paramp1(3))+exp(paramp1(4))+exp(paramp1(5))+exp(paramp1(6))+exp(paramp1(7)));
paramp2 = param_depvarR(34:40);
p_12 = exp(paramp2(1))/(1+exp(paramp2(1))+exp(paramp2(2))+exp(paramp2(3))+exp(paramp2(4))+exp(paramp2(5))+exp(paramp2(6))+exp(paramp2(7)));
p_22 = exp(paramp2(2))/(1+exp(paramp2(1))+exp(paramp2(2))+exp(paramp2(3))+exp(paramp2(4))+exp(paramp2(5))+exp(paramp2(6))+exp(paramp2(7)));
p_32 = exp(paramp2(3))/(1+exp(paramp2(1))+exp(paramp2(2))+exp(paramp2(3))+exp(paramp2(4))+exp(paramp2(5))+exp(paramp2(6))+exp(paramp2(7)));
p_42 = exp(paramp2(4))/(1+exp(paramp2(1))+exp(paramp2(2))+exp(paramp2(3))+exp(paramp2(4))+exp(paramp2(5))+exp(paramp2(6))+exp(paramp2(7)));
p_52 = exp(paramp2(5))/(1+exp(paramp2(1))+exp(paramp2(2))+exp(paramp2(3))+exp(paramp2(4))+exp(paramp2(5))+exp(paramp2(6))+exp(paramp2(7)));
p_62 = exp(paramp2(6))/(1+exp(paramp2(1))+exp(paramp2(2))+exp(paramp2(3))+exp(paramp2(4))+exp(paramp2(5))+exp(paramp2(6))+exp(paramp2(7)));
p_72 = exp(paramp2(7))/(1+exp(paramp2(1))+exp(paramp2(2))+exp(paramp2(3))+exp(paramp2(4))+exp(paramp2(5))+exp(paramp2(6))+exp(paramp2(7)));
p_82 = 1/(1+exp(paramp2(1))+exp(paramp2(2))+exp(paramp2(3))+exp(paramp2(4))+exp(paramp2(5))+exp(paramp2(6))+exp(paramp2(7)));
paramp3 = param_depvarR(41:47);
p_13 = exp(paramp3(1))/(1+exp(paramp3(1))+exp(paramp3(2))+exp(paramp3(3))+exp(paramp3(4))+exp(paramp3(5))+exp(paramp3(6))+exp(paramp3(7)));
p_23 = exp(paramp3(2))/(1+exp(paramp3(1))+exp(paramp3(2))+exp(paramp3(3))+exp(paramp3(4))+exp(paramp3(5))+exp(paramp3(6))+exp(paramp3(7)));
p_33 = exp(paramp3(3))/(1+exp(paramp3(1))+exp(paramp3(2))+exp(paramp3(3))+exp(paramp3(4))+exp(paramp3(5))+exp(paramp3(6))+exp(paramp3(7)));
p_43 = exp(paramp3(4))/(1+exp(paramp3(1))+exp(paramp3(2))+exp(paramp3(3))+exp(paramp3(4))+exp(paramp3(5))+exp(paramp3(6))+exp(paramp3(7)));
p_53 = exp(paramp3(5))/(1+exp(paramp3(1))+exp(paramp3(2))+exp(paramp3(3))+exp(paramp3(4))+exp(paramp3(5))+exp(paramp3(6))+exp(paramp3(7)));
p_63 = exp(paramp3(6))/(1+exp(paramp3(1))+exp(paramp3(2))+exp(paramp3(3))+exp(paramp3(4))+exp(paramp3(5))+exp(paramp3(6))+exp(paramp3(7)));
p_73 = exp(paramp3(7))/(1+exp(paramp3(1))+exp(paramp3(2))+exp(paramp3(3))+exp(paramp3(4))+exp(paramp3(5))+exp(paramp3(6))+exp(paramp3(7)));
p_83 = 1/(1+exp(paramp3(1))+exp(paramp3(2))+exp(paramp3(3))+exp(paramp3(4))+exp(paramp3(5))+exp(paramp3(6))+exp(paramp3(7)));
paramp4 = param_depvarR(48:54);
p_14 = exp(paramp4(1))/(1+exp(paramp4(1))+exp(paramp4(2))+exp(paramp4(3))+exp(paramp4(4))+exp(paramp4(5))+exp(paramp4(6))+exp(paramp4(7)));
p_24 = exp(paramp4(2))/(1+exp(paramp4(1))+exp(paramp4(2))+exp(paramp4(3))+exp(paramp4(4))+exp(paramp4(5))+exp(paramp4(6))+exp(paramp4(7)));
p_34 = exp(paramp4(3))/(1+exp(paramp4(1))+exp(paramp4(2))+exp(paramp4(3))+exp(paramp4(4))+exp(paramp4(5))+exp(paramp4(6))+exp(paramp4(7)));
p_44 = exp(paramp4(4))/(1+exp(paramp4(1))+exp(paramp4(2))+exp(paramp4(3))+exp(paramp4(4))+exp(paramp4(5))+exp(paramp4(6))+exp(paramp4(7)));
p_54 = exp(paramp4(5))/(1+exp(paramp4(1))+exp(paramp4(2))+exp(paramp4(3))+exp(paramp4(4))+exp(paramp4(5))+exp(paramp4(6))+exp(paramp4(7)));
p_64 = exp(paramp4(6))/(1+exp(paramp4(1))+exp(paramp4(2))+exp(paramp4(3))+exp(paramp4(4))+exp(paramp4(5))+exp(paramp4(6))+exp(paramp4(7)));
p_74 = exp(paramp4(7))/(1+exp(paramp4(1))+exp(paramp4(2))+exp(paramp4(3))+exp(paramp4(4))+exp(paramp4(5))+exp(paramp4(6))+exp(paramp4(7)));
p_84 = 1/(1+exp(paramp4(1))+exp(paramp4(2))+exp(paramp4(3))+exp(paramp4(4))+exp(paramp4(5))+exp(paramp4(6))+exp(paramp4(7)));
paramp5 = param_depvarR(55:61);
p_15 = exp(paramp5(1))/(1+exp(paramp5(1))+exp(paramp5(2))+exp(paramp5(3))+exp(paramp5(4))+exp(paramp5(5))+exp(paramp5(6))+exp(paramp5(7)));
p_25 = exp(paramp5(2))/(1+exp(paramp5(1))+exp(paramp5(2))+exp(paramp5(3))+exp(paramp5(4))+exp(paramp5(5))+exp(paramp5(6))+exp(paramp5(7)));
p_35 = exp(paramp5(3))/(1+exp(paramp5(1))+exp(paramp5(2))+exp(paramp5(3))+exp(paramp5(4))+exp(paramp5(5))+exp(paramp5(6))+exp(paramp5(7)));
p_45 = exp(paramp5(4))/(1+exp(paramp5(1))+exp(paramp5(2))+exp(paramp5(3))+exp(paramp5(4))+exp(paramp5(5))+exp(paramp5(6))+exp(paramp5(7)));
p_55 = exp(paramp5(5))/(1+exp(paramp5(1))+exp(paramp5(2))+exp(paramp5(3))+exp(paramp5(4))+exp(paramp5(5))+exp(paramp5(6))+exp(paramp5(7)));
p_65 = exp(paramp5(6))/(1+exp(paramp5(1))+exp(paramp5(2))+exp(paramp5(3))+exp(paramp5(4))+exp(paramp5(5))+exp(paramp5(6))+exp(paramp5(7)));
p_75 = exp(paramp5(7))/(1+exp(paramp5(1))+exp(paramp5(2))+exp(paramp5(3))+exp(paramp5(4))+exp(paramp5(5))+exp(paramp5(6))+exp(paramp5(7)));
p_85 = 1/(1+exp(paramp5(1))+exp(paramp5(2))+exp(paramp5(3))+exp(paramp5(4))+exp(paramp5(5))+exp(paramp5(6))+exp(paramp5(7)));
paramp6 = param_depvarR(62:68);
p_16 = exp(paramp6(1))/(1+exp(paramp6(1))+exp(paramp6(2))+exp(paramp6(3))+exp(paramp6(4))+exp(paramp6(5))+exp(paramp6(6))+exp(paramp6(7)));
p_26 = exp(paramp6(2))/(1+exp(paramp6(1))+exp(paramp6(2))+exp(paramp6(3))+exp(paramp6(4))+exp(paramp6(5))+exp(paramp6(6))+exp(paramp6(7)));
p_36 = exp(paramp6(3))/(1+exp(paramp6(1))+exp(paramp6(2))+exp(paramp6(3))+exp(paramp6(4))+exp(paramp6(5))+exp(paramp6(6))+exp(paramp6(7)));
p_46 = exp(paramp6(4))/(1+exp(paramp6(1))+exp(paramp6(2))+exp(paramp6(3))+exp(paramp6(4))+exp(paramp6(5))+exp(paramp6(6))+exp(paramp6(7)));
p_56 = exp(paramp6(5))/(1+exp(paramp6(1))+exp(paramp6(2))+exp(paramp6(3))+exp(paramp6(4))+exp(paramp6(5))+exp(paramp6(6))+exp(paramp6(7)));
p_66 = exp(paramp6(6))/(1+exp(paramp6(1))+exp(paramp6(2))+exp(paramp6(3))+exp(paramp6(4))+exp(paramp6(5))+exp(paramp6(6))+exp(paramp6(7)));
p_76 = exp(paramp6(7))/(1+exp(paramp6(1))+exp(paramp6(2))+exp(paramp6(3))+exp(paramp6(4))+exp(paramp6(5))+exp(paramp6(6))+exp(paramp6(7)));
p_86 = 1/(1+exp(paramp6(1))+exp(paramp6(2))+exp(paramp6(3))+exp(paramp6(4))+exp(paramp6(5))+exp(paramp6(6))+exp(paramp6(7)));
paramp7 = param_depvarR(69:75);
p_17 = exp(paramp7(1))/(1+exp(paramp7(1))+exp(paramp7(2))+exp(paramp7(3))+exp(paramp7(4))+exp(paramp7(5))+exp(paramp7(6))+exp(paramp7(7)));
p_27 = exp(paramp7(2))/(1+exp(paramp7(1))+exp(paramp7(2))+exp(paramp7(3))+exp(paramp7(4))+exp(paramp7(5))+exp(paramp7(6))+exp(paramp7(7)));
p_37 = exp(paramp7(3))/(1+exp(paramp7(1))+exp(paramp7(2))+exp(paramp7(3))+exp(paramp7(4))+exp(paramp7(5))+exp(paramp7(6))+exp(paramp7(7)));
p_47 = exp(paramp7(4))/(1+exp(paramp7(1))+exp(paramp7(2))+exp(paramp7(3))+exp(paramp7(4))+exp(paramp7(5))+exp(paramp7(6))+exp(paramp7(7)));
p_57 = exp(paramp7(5))/(1+exp(paramp7(1))+exp(paramp7(2))+exp(paramp7(3))+exp(paramp7(4))+exp(paramp7(5))+exp(paramp7(6))+exp(paramp7(7)));
p_67 = exp(paramp7(6))/(1+exp(paramp7(1))+exp(paramp7(2))+exp(paramp7(3))+exp(paramp7(4))+exp(paramp7(5))+exp(paramp7(6))+exp(paramp7(7)));
p_77 = exp(paramp7(7))/(1+exp(paramp7(1))+exp(paramp7(2))+exp(paramp7(3))+exp(paramp7(4))+exp(paramp7(5))+exp(paramp7(6))+exp(paramp7(7)));
p_87 = 1/(1+exp(paramp7(1))+exp(paramp7(2))+exp(paramp7(3))+exp(paramp7(4))+exp(paramp7(5))+exp(paramp7(6))+exp(paramp7(7)));
paramp8 = param_depvarR(76:82);
p_18 = exp(paramp8(1))/(1+exp(paramp8(1))+exp(paramp8(2))+exp(paramp8(3))+exp(paramp8(4))+exp(paramp8(5))+exp(paramp8(6))+exp(paramp8(7)));
p_28 = exp(paramp8(2))/(1+exp(paramp8(1))+exp(paramp8(2))+exp(paramp8(3))+exp(paramp8(4))+exp(paramp8(5))+exp(paramp8(6))+exp(paramp8(7)));
p_38 = exp(paramp8(3))/(1+exp(paramp8(1))+exp(paramp8(2))+exp(paramp8(3))+exp(paramp8(4))+exp(paramp8(5))+exp(paramp8(6))+exp(paramp8(7)));
p_48 = exp(paramp8(4))/(1+exp(paramp8(1))+exp(paramp8(2))+exp(paramp8(3))+exp(paramp8(4))+exp(paramp8(5))+exp(paramp8(6))+exp(paramp8(7)));
p_58 = exp(paramp8(5))/(1+exp(paramp8(1))+exp(paramp8(2))+exp(paramp8(3))+exp(paramp8(4))+exp(paramp8(5))+exp(paramp8(6))+exp(paramp8(7)));
p_68 = exp(paramp8(6))/(1+exp(paramp8(1))+exp(paramp8(2))+exp(paramp8(3))+exp(paramp8(4))+exp(paramp8(5))+exp(paramp8(6))+exp(paramp8(7)));
p_78 = exp(paramp8(7))/(1+exp(paramp8(1))+exp(paramp8(2))+exp(paramp8(3))+exp(paramp8(4))+exp(paramp8(5))+exp(paramp8(6))+exp(paramp8(7)));
p_88 = 1/(1+exp(paramp8(1))+exp(paramp8(2))+exp(paramp8(3))+exp(paramp8(4))+exp(paramp8(5))+exp(paramp8(6))+exp(paramp8(7)));

% Transition probability matrix of Sm, Sf and SsR
PSmfsR = [p_11	p_12	p_13	p_14	p_15	p_16	p_17	p_18;
          p_21	p_22	p_23	p_24	p_25	p_26	p_27	p_28;
          p_31	p_32	p_33	p_34	p_35	p_36	p_37	p_38;
          p_41	p_42	p_43	p_44	p_45	p_46	p_47	p_48;
          p_51	p_52	p_53	p_54	p_55	p_56	p_57	p_58;
          p_61	p_62	p_63	p_64	p_65	p_66	p_67	p_68;
          p_71	p_72	p_73	p_74	p_75	p_76	p_77	p_78;
          p_81	p_82	p_83	p_84	p_85	p_86	p_87	p_88];
 
% Transition Probability matrix variance of tau
pstau_11 = exp(param_depvarR(83))/(1+exp(param_depvarR(83)));
pstau_21 = 1/(1+exp(param_depvarR(83)));
pstau_12 = exp(param_depvarR(84))/(1+exp(param_depvarR(84)));
pstau_22 = 1/(1+exp(param_depvarR(84)));
PSstau = [pstau_11 pstau_12;
          pstau_21 pstau_22];
   
% Global Transition Probability Matrix
P = kron(PSstau,PSmfsR);

A = [eye(16)-P;ones(1,16)];
p_ = (A'*A)\A';
p = p_(:,17); % unconditional probabilities vector

% Obtaining marginal transition probabilities
PSmf = marginalprobabilities8to4(PSmfsR,1,1,0);
PSmsR = marginalprobabilities8to4(PSmfsR,1,0,1);
[PSm,PSf] = marginalprobabilities4(PSmf);
[~,PSsR] = marginalprobabilities4(PSmsR);
save('P','P');
save('PSmf','PSmf');
save('PSmsR','PSmsR');
save('PSm','PSm');
save('PSf','PSf');
save('PSsR','PSsR');

%% Computing V-C matrix
param_mean = [R_1;R_2;alphap_1;alphap_2-alphap_1;alphay_1;alphay_2-alphay_1;kappaRp_1;kappaRp_2-kappaRp_1;kappaRy_1;kappaRy_2-kappaRy_1;rhoR_1;rhoR_2;sigmaR_1;sigmaR_2-sigmaR_1;...
              tau_1;tau_2;gammab_1;gammab_2-gammab_1;gammay_1;gammay_2-gammay_1;kappatauy_1;kappatauy_2-kappatauy_1;rhotau_1;rhotau_2;sigmatau_1;sigmatau_2-sigmatau_1];
[jacobianparam,err] = jacobianest(@(param)fun_vec_param(param),param_depvarR(1:size(paramm,1)));
varianceparam = diag(jacobianparam*(hessian(1:26,1:26)\jacobianparam'));
tstatparam = param_mean./sqrt(varianceparam); % t-statisctics

%% Coefficients by Regime
Taylor_11 = [R_1;alphap_1;alphay_1;kappaRp_1;kappaRy_1;rhoR_1;sigmaR_1]; % jm = 1 jsR = 1
Taylor_21 = [R_2;alphap_2;alphay_2;kappaRp_2;kappaRy_2;rhoR_2;sigmaR_1]; % jm = 2 jsR = 1
Taylor_12 = [R_1;alphap_1;alphay_1;kappaRp_1;kappaRy_1;rhoR_1;sigmaR_2]; % jm = 1 jsR = 2
Taylor_22 = [R_2;alphap_2;alphay_2;kappaRp_2;kappaRy_2;rhoR_2;sigmaR_2]; % jm = 2 jsR = 2
Tax_11 = [tau_1;gammab_1;gammay_1;kappatauy_1;rhotau_1;sigmatau_1]; % jf = 1 jstau = 1
Tax_21 = [tau_2;gammab_2;gammay_2;kappatauy_2;rhotau_2;sigmatau_1]; % jf = 2 jstau = 1
Tax_12 = [tau_1;gammab_1;gammay_1;kappatauy_1;rhotau_1;sigmatau_2]; % jf = 1 jstau = 2
Tax_22 = [tau_2;gammab_2;gammay_2;kappatauy_2;rhotau_2;sigmatau_2]; % jf = 2 jstau = 2

%% Obtaining smoothed probablities
[pred_prob, filter_prob, smooth_prob] = estimatedprob_depvarR(param_depvarR,data);

prob_jm_2_s_depvarR = smooth_prob(:,2)+smooth_prob(:,4) + smooth_prob(:,6)+smooth_prob(:,8) + smooth_prob(:,10)+smooth_prob(:,12) + smooth_prob(:,14)+smooth_prob(:,16);
prob_jf_2_s_depvarR = sum(smooth_prob(:,3:4),2) + sum(smooth_prob(:,7:8),2)+ sum(smooth_prob(:,11:12),2) + sum(smooth_prob(:,15:16),2);
prob_jsR_2_s_depvarR = sum(smooth_prob(:,5:8),2)+sum(smooth_prob(:,13:16),2);
prob_jstau_2_s_depvarR = sum(smooth_prob(:,9:16),2);

prob_M_regime_s_depvarR = smooth_prob(:,4)+smooth_prob(:,8) + smooth_prob(:,12)+smooth_prob(:,16);  % According to the results, this holds when jm = 2 and jf = 2
prob_F_regime_s_depvarR = smooth_prob(:,1)+smooth_prob(:,5) + smooth_prob(:,9)+smooth_prob(:,13);  % According to the results, this holds when jm = 1 and jf = 1
prob_I_regime_s_depvarR = smooth_prob(:,3)+smooth_prob(:,7) + smooth_prob(:,11)+smooth_prob(:,15);  % According to the results, this holds when jm = 1 and jf = 2
prob_E_regime_s_depvarR = smooth_prob(:,2)+smooth_prob(:,6) + smooth_prob(:,10)+smooth_prob(:,14);  % According to the results, this holds when jm = 2 and jf = 1

prob_jm_2_f_depvarR = filter_prob(:,2)+filter_prob(:,4) + filter_prob(:,6)+filter_prob(:,8) + filter_prob(:,10)+filter_prob(:,12) + filter_prob(:,14)+filter_prob(:,16);
prob_jf_2_f_depvarR = sum(filter_prob(:,3:4),2) + sum(filter_prob(:,7:8),2)+ sum(filter_prob(:,11:12),2) + sum(filter_prob(:,15:16),2);
prob_jsR_2_f_depvarR = sum(filter_prob(:,5:8),2)+sum(filter_prob(:,13:16),2);
prob_jstau_2_f_depvarR = sum(filter_prob(:,9:16),2);

prob_M_regime_f_depvarR = filter_prob(:,4)+filter_prob(:,8) + filter_prob(:,12)+filter_prob(:,16);  % According to the results, this holds when jm = 2 and jf = 2
prob_F_regime_f_depvarR = filter_prob(:,1)+filter_prob(:,5) + filter_prob(:,9)+filter_prob(:,13);  % According to the results, this holds when jm = 1 and jf = 1
prob_I_regime_f_depvarR = filter_prob(:,3)+filter_prob(:,7) + filter_prob(:,11)+filter_prob(:,15);  % According to the results, this holds when jm = 1 and jf = 2
prob_E_regime_f_depvarR = filter_prob(:,2)+filter_prob(:,6) + filter_prob(:,10)+filter_prob(:,14);  % According to the results, this holds when jm = 2 and jf = 1

save('prob_jm_2_s_depvarR','prob_jm_2_s_depvarR');
save('prob_jf_2_s_depvarR','prob_jf_2_s_depvarR');
save('prob_jsR_2_s_depvarR','prob_jsR_2_s_depvarR');
save('prob_jstau_2_s_depvarR','prob_jstau_2_s_depvarR');

%% Figures
t = (1961.5:.25:2015.5)';

prob_M_regime_s = prob_M_regime_s_depvarR;
prob_F_regime_s = prob_F_regime_s_depvarR;
prob_I_regime_s = prob_I_regime_s_depvarR;
prob_E_regime_s = prob_E_regime_s_depvarR;
prob_jsR_2_s = prob_jsR_2_s_depvarR;
prob_jstau_2_s = prob_jstau_2_s_depvarR;

figure1 = figure;

% Create subplot
subplot1 = subplot(4,1,1,'Parent',figure1,'YTick',[0 0.5 1]);
xlim(subplot1,[1961.5 2015.5]);
ylim(subplot1,[0 1]);
box(subplot1,'on');
hold(subplot1,'all');

% Create plot
area(t,prob_M_regime_s,'Parent',subplot1,'LineWidth',2.5,'DisplayName','Prob(Regime M)','EdgeColor',[0 0 1],'FaceColor',[0 0 1]);

% Create subplot
subplot2 = subplot(4,1,2,'Parent',figure1,'YTick',[0 0.5 1]);
xlim(subplot2,[1961.5 2015.5]);
ylim(subplot2,[0 1]);
box(subplot2,'on');
hold(subplot2,'all');

% Create plot
area(t,prob_F_regime_s,'Parent',subplot2,'LineWidth',2.5,'DisplayName','Prob(Regime F)','EdgeColor',[0 1 0],'FaceColor', [0 1 0]);

% Create subplot
subplot3 = subplot(4,1,3,'Parent',figure1,'YTick',[0 0.5 1]);
xlim(subplot3,[1961.5 2015.5]);
ylim(subplot3,[0 1]);
box(subplot3,'on');
hold(subplot3,'all');

% Create plot
area(t,prob_I_regime_s,'Parent',subplot3,'LineWidth',2.5,'DisplayName','Prob(Regime I)','EdgeColor',[1 1 0],'FaceColor', [1 1 0]);

% Create subplot
subplot4 = subplot(4,1,4,'Parent',figure1,'YTick',[0 0.5 1]);
xlim(subplot4,[1961.5 2015.5]);
ylim(subplot4,[0 1]);
box(subplot4,'on');
hold(subplot4,'all');

% Create plot
area(t,prob_E_regime_s,'Parent',subplot4,'LineWidth',2.5,'DisplayName','Prob(Regime E)','EdgeColor',[1 0 0],'FaceColor', [1 0 0]);

% Create legend
legend1 = legend(subplot4,'show');
set(legend1,'Position',[0.617173291764093 0.197648376259798 0.112236944660951 0.0261291526689063],'EdgeColor',[1 1 1]);

% Create legend
legend2 = legend(subplot3,'show');
set(legend2,'Position',[0.617173291764093 0.426091825307951 0.112236944660951 0.0261291526689063],'EdgeColor',[1 1 1]);

% Create legend
legend3 = legend(subplot2,'show');
set(legend3,'Position',[0.617173291764093 0.66013437849944 0.116134060795012 0.0261291526689063],'EdgeColor',[1 1 1]);

% Create legend
legend4 = legend(subplot1,'show');
set(legend4,'Position',[0.246167835801506 0.882978723404255 0.118472330475448 0.0261291526689063],'EdgeColor',[1 1 1]);

%%

% Create figure
figure2 = figure;

% Create subplot
subplot21 = subplot(2,1,1,'Parent',figure2,'YTick',[0 0.5 1]);
xlim(subplot21,[1961.5 2015.5]);
ylim(subplot21,[0 1]);
box(subplot21,'on');
hold(subplot21,'all');

% Create plot
area(t,prob_jsR_2_s,'Parent',subplot21,'LineWidth',2.5,'DisplayName','Prob(High \sigma_R)',...
    'FaceColor',[0.501960813999176 0.501960813999176 0.501960813999176],...
    'EdgeColor',[0.501960813999176 0.501960813999176 0.501960813999176],...
    'LineWidth',2.5);
% Create subplot
subplot22 = subplot(2,1,2,'Parent',figure2,'YTick',[0 0.5 1]);
xlim(subplot22,[1961.5 2015.5]);
ylim(subplot22,[0 1]);
box(subplot22,'on');
hold(subplot22,'all');

% Create plot
area(t,prob_jstau_2_s,'Parent',subplot22,'LineWidth',2.5,'DisplayName','Prob(High \sigma_\tau)',...
    'FaceColor',[0.501960813999176 0.501960813999176 0.501960813999176],...
    'EdgeColor',[0.501960813999176 0.501960813999176 0.501960813999176],...
    'LineWidth',2.5);

% Create legend
legend21 = legend(subplot22,'show');
set(legend21,'Position',[0.578576010002594 0.345516427759803 0.106780982073266 0.0328480776409108],'EdgeColor',[1 1 1]);

% Create legend
legend22 = legend(subplot21,'show');
set(legend22,'Position',[0.578576010002594 0.822242654339003 0.106780982073266 0.0328480776409108],'EdgeColor',[1 1 1]);

%% Display Results
display('Parameter Estimates');
[[{'R_c'};
 {'alphap_1'};
 {'alphap_2'};
 {'alphay_1'};
 {'alphay_2'};
 {'kappaRp_1'};
 {'kappaRp_2'};
 {'kappaRy_1'};
 {'kappaRy_2'};
 {'rhoR'};
 {'sigmaR_1'};
 {'sigmaR_2'};
 {'tau_c'};
 {'gammab_1'};
 {'gammab_2'};
 {'gammay_1'};
 {'gammay_2'};
 {'kappatauy_1'};
 {'kappatauy_2'};
 {'rhotau'};
 {'sigmatau_1'};
 {'sigmatau_2'};
 {'pmf_11'};
 {'pmf_21'};
 {'pmf_31'};
 {'pmf_12'};
 {'pmf_22'};
 {'pmf_32'};
 {'pmf_13'};
 {'pmf_23'};
 {'pmf_33'};
 {'pmf_14'};
 {'pmf_24'};
 {'pmf_34'};
 {'psR_11'};
 {'psR_12'};
 {'pstau_11'};
 {'pstau_12'}] [R_c;alphap_1;alphap_2;alphay_1;alphay_2;kappaRp_1;kappaRp_2;kappaRy_1;kappaRy_2;rhoR;sigmaR_1;sigmaR_2;...
              tau_c;gammab_1;gammab_2;gammay_1;gammay_2;kappatauy_1;kappatauy_2;rhotau;sigmatau_1;sigmatau_2;...
              pmf_11;pmf_21;pmf_31;pmf_12;pmf_22;pmf_32;pmf_13;pmf_23;pmf_33;pmf_14;pmf_24;pmf_34;...
              psR_11;psR_12;pstau_11;pstau_12] tstatparam]

%% Forecasting Interest Rate
betaR_11 = [rhoR_1 (1-rhoR_1)*[R_1 alphap_1 alphay_1] sigmaR_1*kappaRp_1 sigmaR_1*kappaRy_1]';
betaR_12 = [rhoR_1 (1-rhoR_1)*[R_1 alphap_1 alphay_1] sigmaR_2*kappaRp_1 sigmaR_2*kappaRy_1]';
betaR_21 = [rhoR_2 (1-rhoR_2)*[R_2 alphap_2 alphay_2] sigmaR_1*kappaRp_2 sigmaR_1*kappaRy_2]';
betaR_22 = [rhoR_2 (1-rhoR_2)*[R_2 alphap_2 alphay_2] sigmaR_2*kappaRp_2 sigmaR_2*kappaRy_2]';

T = length(data);
R = data(2:T,1);
dzlb = (R==0.25); % Dummy ZLB
xR = [data(1:T-1,1) ones(T-1,1) data(2:T,3:4) data(2:T,6:7)];

prob_jm_2_f = filter_prob(:,2)+filter_prob(:,4) + filter_prob(:,6)+filter_prob(:,8) + filter_prob(:,10)+filter_prob(:,12) + filter_prob(:,14)+filter_prob(:,16);
prob_jsR_2_f = sum(filter_prob(:,5:8),2)+sum(filter_prob(:,13:16),2);

Rpredicted = (1-prob_jm_2_f).*(1-prob_jsR_2_f).*(xR*betaR_11) + (1-prob_jm_2_f).*prob_jsR_2_f.*(xR*betaR_12) + prob_jm_2_f.*(1-prob_jsR_2_f).*(xR*betaR_21) + prob_jm_2_f.*prob_jsR_2_f.*(xR*betaR_22);

figure3 = figure;
axes3 = axes('Parent',figure3);
xlim(axes3,[1961.5 2014]);
box(axes3,'on');
hold(axes3,'all');
plot31 = plot(t,R,'LineWidth',2.5,'LineWidth',2.5,'Color',[0 0 0]);
hold on
plot32 = plot(t,Rpredicted,'LineWidth',2.5,'LineStyle','--','Color',[0.313725501298904 0.313725501298904 0.313725501298904]);
plot3 = plot(t,zeros(size(t,1),1),'LineWidth',1,'LineStyle',':','Color',[0 0 0]);
legend31 = legend([plot31 plot32], 'R_t', 'R_t Predicted');
set(legend31,'YColor',[1 1 1],'XColor',[1 1 1],...
    'Position',[0.640555988568459 0.657708100037327 0.116913484021824 0.0761478163493841],...
    'FontSize',14,...
    'FontName','Garamond');

%% Calculating Correlation Between Sm and Sf
A = [eye(4)-PSmf;ones(1,4)];
p_ = (A'*A)\A';
p = p_(:,5);
varm = (p(1)+p(3))*(1-p(1)-p(3));
varf = (p(1)+p(2))*(1-p(1)-p(2));
covmf = p(1)*p(4)-p(2)*p(3);
corrmf = covmf/sqrt(varm*varf);