%% This code estimates the Taylor and fiscal rules with M-S coefficients
% jointly to obtain the probability of a hawkish monetary policy regime
% IN THIS ESTIMATION, THE SWITCHING OF THE VARIANCE OF THE INTEREST RATE
% EQUATION IS NOT ASSUMED TO BE CORRELATED WITH THE OTHER SWITCHING STATES, 
% EXCEPT THE SWITCHING STATE OF TAXES.
%
% The policy rules are specified as
%
% Rstar_t = rho_R*Rstar_{t-1} + (1-rho_R)*[Rstar + alpha^p(S_{mt})*p_t + alpha^y(S_{mt})*y_t] + sigma(S_{sRt})e_Rt, e_Rt ~ N(0,1)
% R_t = max(Rstar_t,0.25)
%
% tau_t = rho_tau*tau_{t-1} + (1-rho_tau)*[tau + gamma^b(S_{ft})*b_{t-1} + gamma^y(S_{ft})*y_t] + sigma(S_{staut})e_taut, e_taut ~ N(0,1)
%
% where
%
% S_{mt} = 1,2,...,J_m
% S_{ft} = 1,2,...,J_f
% S_{sRt} = 1,2,...,J_sR
% S_{staut} = 1,2,...,J_stau
% J_m = 2
% J_f = 2
% J_sR = 2
% J_stau = 2
% 
% S_t = S_{mt} + (S_{ft}-1)J_f = 1,2,...,J
% J = J_m + (J_f - 1)J_f = 4
%
% P(S_t = j|S_{t-1} = j') = p_{jj'}
% P(S_{sRt} = jsR|S_{sRt-1} = jsR') = p_{jsRjsR'}
% P(S_{staut} = jstau|S_{staut-1} = jstau') = p_{jstaujstau'}
%
% It is assumed that p_t and y_t are endogenous and an endogeneity
% correction has to be made. There are also two breaks in the tax series:
% 1960:1-1969:2 and 2001:2-2013:1
% 
% 
% Manuel Gonzalez-Astudill0
% 1/9/2017
%% Loading Data
clc,clear;
% data = [R_t tau_t p_t y_t b_{t-1} v_p v_y]
data = load('data_new_pc_net.txt');
vc_upuy = load('vc_upuy_new_pc_net.txt');
Sigmasqrt = chol(vc_upuy); % upper triangular
v = data(:,6:7)/Sigmasqrt; % Standardized Residuals
data(:,6:7) = v;

PSmf = [0.94 0    0    0.07;
        0    0    0.16 0;
        0.06 0.46 0    0;
        0    0.54 0.84 0.93];
PSsR = [0.92 0.17;
        0.08 0.83];
PSstau = [0.93 0.4;
          0.07 0.6];
%% Setting Maximum Likelihood Estimation
param0 = [log(2.57);  log(0.73); log(0.89); log(1.87); -1.65; -0.06; -0.27; -0.03; 0.12; log(9); log(0.23); log(1.02);...
          log(1.77); 0.07; log(0.06); log(0.34); -0.26; 0.27; 0.09; log(9); log(0.09); log(0.48);...
          transformedprobabilities(PSmf);...
          transformedprobabilities(PSsR);...
          transformedprobabilities(PSstau)];
options = optimoptions('fminunc','Diagnostics','on','Algorithm','quasi-newton','Display','iter','FunValCheck','off','MaxFunEvals',100000,'MaxIter',10000);
[paramfixedrhoint,fval,exitflag,output,grad,hess] = fminunc(@(param)loglikfunfixedrhoint(param,data),param0,options);
save('paramfixedrhoint','paramfixedrhoint');
%% Obtaining Results
paramm = paramfixedrhoint(1:22);
R_c = exp(paramm(1));
alphap_1 = exp(paramm(2));
alphap_2 = alphap_1 + exp(paramm(3));
alphay_1 = exp(paramm(4));
alphay_2 = alphay_1 + paramm(5);
kappaRp_1 = paramm(6);
kappaRp_2 = kappaRp_1 + paramm(7);
kappaRy_1 = paramm(8);
kappaRy_2 = kappaRy_1 + paramm(9);
rhoR = exp(paramm(10))/(1+exp(paramm(10)));
sigmaR_1 = exp(paramm(11));
sigmaR_2 = sigmaR_1 + exp(paramm(12));
tau_c = exp(paramm(13));
gammab_1 = paramm(14);
gammab_2 = gammab_1 + exp(paramm(15));
gammay_1 = exp(paramm(16));
gammay_2 = gammay_1 + paramm(17);
kappatauy_1 = paramm(18);
kappatauy_2 = kappatauy_1 + paramm(19);
rhotau = exp(paramm(20))/(1+exp(paramm(20)));
sigmatau_1 = exp(paramm(21));
sigmatau_2 = sigmatau_1 + exp(paramm(22));


betatau_11 = [rhotau (1-rhotau)*[tau_c gammab_1 gammay_1] sigmatau_1*kappatauy_1]';
betatau_12 = [rhotau (1-rhotau)*[tau_c gammab_1 gammay_1] sigmatau_2*kappatauy_1]';
betatau_21 = [rhotau (1-rhotau)*[tau_c gammab_2 gammay_2] sigmatau_1*kappatauy_2]';
betatau_22 = [rhotau (1-rhotau)*[tau_c gammab_2 gammay_2] sigmatau_2*kappatauy_2]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Smf
% jmf = 1 <-- jm = 1, jf = 1;
% jmf = 2 <-- jm = 2, jf = 1;
% jmf = 3 <-- jm = 1, jf = 2;
% jmf = 4 <-- jm = 2, jf = 2;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
paramp = paramfixedrhoint(23:38);
pmf_11 = exp(paramp(1))/(1+exp(paramp(1))+exp(paramp(2))+exp(paramp(3)));
pmf_21 = exp(paramp(2))/(1+exp(paramp(1))+exp(paramp(2))+exp(paramp(3)));
pmf_31 = exp(paramp(3))/(1+exp(paramp(1))+exp(paramp(2))+exp(paramp(3)));
pmf_41 = 1/(1+exp(paramp(1))+exp(paramp(2))+exp(paramp(3)));
pmf_12 = exp(paramp(4))/(1+exp(paramp(4))+exp(paramp(5))+exp(paramp(6)));
pmf_22 = exp(paramp(5))/(1+exp(paramp(4))+exp(paramp(5))+exp(paramp(6)));
pmf_32 = exp(paramp(6))/(1+exp(paramp(4))+exp(paramp(5))+exp(paramp(6)));
pmf_42 = 1/(1+exp(paramp(4))+exp(paramp(5))+exp(paramp(6)));
pmf_13 = exp(paramp(7))/(1+exp(paramp(7))+exp(paramp(8))+exp(paramp(9)));
pmf_23 = exp(paramp(8))/(1+exp(paramp(7))+exp(paramp(8))+exp(paramp(9)));
pmf_33 = exp(paramp(9))/(1+exp(paramp(7))+exp(paramp(8))+exp(paramp(9)));
pmf_43 = 1/(1+exp(paramp(7))+exp(paramp(8))+exp(paramp(9)));
pmf_14 = exp(paramp(10))/(1+exp(paramp(10))+exp(paramp(11))+exp(paramp(12)));
pmf_24 = exp(paramp(11))/(1+exp(paramp(10))+exp(paramp(11))+exp(paramp(12)));
pmf_34 = exp(paramp(12))/(1+exp(paramp(10))+exp(paramp(11))+exp(paramp(12)));
pmf_44 = 1/(1+exp(paramp(10))+exp(paramp(11))+exp(paramp(12)));

PSmf = [pmf_11 pmf_12 pmf_13 pmf_14;
        pmf_21 pmf_22 pmf_23 pmf_24;
        pmf_31 pmf_32 pmf_33 pmf_34;
        pmf_41 pmf_42 pmf_43 pmf_44];

% Transition Probabilities for standard deviation of R
psR_11 = exp(paramp(13))/(1+exp(paramp(13)));
psR_21 = 1/(1+exp(paramp(13)));
psR_12 = exp(paramp(14))/(1+exp(paramp(14)));
psR_22 = 1/(1+exp(paramp(14)));

PSsR = [psR_11 psR_12;
       psR_21 psR_22];
   
% Transition Probabilities for standard deviation of tau
pstau_11 = exp(paramp(15))/(1+exp(paramp(15)));
pstau_21 = 1/(1+exp(paramp(15)));
pstau_12 = exp(paramp(16))/(1+exp(paramp(16)));
pstau_22 = 1/(1+exp(paramp(16)));

PSstau = [pstau_11 pstau_12;
         pstau_21 pstau_22];
   
% Global Transition Probability Matrix
P = kron(PSstau,kron(PSsR,PSmf));

A = [eye(16)-P;ones(1,16)];
p_ = (A'*A)\A';
p = p_(:,17); % unconditional probabilities vector

% Obtaining marginal transition probabilities
[PSm,PSf] = marginalprobabilities4(PSmf);
save('P','P');
save('PSmf','PSmf');
save('PSm','PSm');
save('PSf','PSf');
save('PSsR','PSsR');

%% Computing V-C matrix
param_mean = [R_c;alphap_1;alphap_2-alphap_1;alphay_1;alphay_2-alphay_1;kappaRp_1;kappaRp_2;kappaRy_1;kappaRy_2;rhoR;sigmaR_1;sigmaR_2-sigmaR_1;...
              tau_c;gammab_1;gammab_2-gammab_1;gammay_1;gammay_2-gammay_1;kappatauy_1;kappatauy_2;rhotau;sigmatau_1;sigmatau_2-sigmatau_1;
              pmf_11;pmf_21;pmf_31;pmf_12;pmf_22;pmf_32;pmf_13;pmf_23;pmf_33;pmf_14;pmf_24;pmf_34;
              psR_11;psR_12;pstau_11;pstau_12];
[jacobianparam,err] = jacobianest(@(param)fun_vec_param_fixedrhoint(param),paramfixedrhoint);
varianceparam = abs(diag(jacobianparam*(hess\jacobianparam')));
tstatparam = param_mean./sqrt(varianceparam); % t-statisctics

%% Coefficients by Regime
Taylor_11 = [R_c;alphap_1;alphay_1;kappaRp_1;kappaRy_1;rhoR;sigmaR_1]; % jm = 1 jsR = 1
Taylor_21 = [R_c;alphap_2;alphay_2;kappaRp_2;kappaRy_2;rhoR;sigmaR_1]; % jm = 2 jsR = 1
Taylor_12 = [R_c;alphap_1;alphay_1;kappaRp_1;kappaRy_1;rhoR;sigmaR_2]; % jm = 1 jsR = 2
Taylor_22 = [R_c;alphap_2;alphay_2;kappaRp_2;kappaRy_2;rhoR;sigmaR_2]; % jm = 2 jsR = 2
Tax_11 = [tau_c;gammab_1;gammay_1;kappatauy_1;rhotau;sigmatau_1]; % jf = 1 jstau = 1
Tax_21 = [tau_c;gammab_2;gammay_2;kappatauy_2;rhotau;sigmatau_1]; % jf = 2 jstau = 1
Tax_12 = [tau_c;gammab_1;gammay_1;kappatauy_1;rhotau;sigmatau_2]; % jf = 1 jstau = 2
Tax_22 = [tau_c;gammab_2;gammay_2;kappatauy_2;rhotau;sigmatau_2]; % jf = 2 jstau = 2

%% Obtaining smoothed probablities
[pred_prob, filter_prob, smooth_prob] = estimatedprobfixedrhoint(paramfixedrhoint,data);

prob_jm_2_s = smooth_prob(:,2)+smooth_prob(:,4) + smooth_prob(:,6)+smooth_prob(:,8) + smooth_prob(:,10)+smooth_prob(:,12) + smooth_prob(:,14)+smooth_prob(:,16);
prob_jf_2_s = sum(smooth_prob(:,3:4),2) + sum(smooth_prob(:,7:8),2)+ sum(smooth_prob(:,11:12),2) + sum(smooth_prob(:,15:16),2);
prob_jsR_2_s = sum(smooth_prob(:,5:8),2)+sum(smooth_prob(:,13:16),2);
prob_jstau_2_s = sum(smooth_prob(:,9:16),2);

prob_M_regime_s = smooth_prob(:,4)+smooth_prob(:,8) + smooth_prob(:,12)+smooth_prob(:,16);  % According to the results, this holds when jm = 2 and jf = 2
prob_F_regime_s = smooth_prob(:,1)+smooth_prob(:,5) + smooth_prob(:,9)+smooth_prob(:,13);  % According to the results, this holds when jm = 1 and jf = 1
prob_I_regime_s = smooth_prob(:,3)+smooth_prob(:,7) + smooth_prob(:,11)+smooth_prob(:,15);  % According to the results, this holds when jm = 1 and jf = 2
prob_E_regime_s = smooth_prob(:,2)+smooth_prob(:,6) + smooth_prob(:,10)+smooth_prob(:,14);  % According to the results, this holds when jm = 2 and jf = 1

prob_jm_2_f = filter_prob(:,2)+filter_prob(:,4) + filter_prob(:,6)+filter_prob(:,8) + filter_prob(:,10)+filter_prob(:,12) + filter_prob(:,14)+filter_prob(:,16);
prob_jf_2_f = sum(filter_prob(:,3:4),2) + sum(filter_prob(:,7:8),2)+ sum(filter_prob(:,11:12),2) + sum(filter_prob(:,15:16),2);
prob_jsR_2_f = sum(filter_prob(:,5:8),2)+sum(filter_prob(:,13:16),2);
prob_jstau_2_f = sum(filter_prob(:,9:16),2);

prob_M_regime_f = filter_prob(:,4)+filter_prob(:,8) + filter_prob(:,12)+filter_prob(:,16);  % According to the results, this holds when jm = 2 and jf = 2
prob_F_regime_f = filter_prob(:,1)+filter_prob(:,5) + filter_prob(:,9)+filter_prob(:,13);  % According to the results, this holds when jm = 1 and jf = 1
prob_I_regime_f = filter_prob(:,3)+filter_prob(:,7) + filter_prob(:,11)+filter_prob(:,15);  % According to the results, this holds when jm = 1 and jf = 2
prob_E_regime_f = filter_prob(:,2)+filter_prob(:,6) + filter_prob(:,10)+filter_prob(:,14);  % According to the results, this holds when jm = 2 and jf = 1

prob_M_regime_p = pred_prob(:,4)+pred_prob(:,8) + pred_prob(:,12)+pred_prob(:,16);  % According to the results, this holds when jm = 2 and jf = 2
prob_F_regime_p = pred_prob(:,1)+pred_prob(:,5) + pred_prob(:,9)+pred_prob(:,13);  % According to the results, this holds when jm = 1 and jf = 1
prob_I_regime_p = pred_prob(:,3)+pred_prob(:,7) + pred_prob(:,11)+pred_prob(:,15);  % According to the results, this holds when jm = 1 and jf = 2
prob_E_regime_p = pred_prob(:,2)+pred_prob(:,6) + pred_prob(:,10)+pred_prob(:,14);  % According to the results, this holds when jm = 2 and jf = 1


save('prob_jm_2_s','prob_jm_2_s');
save('prob_jf_2_s','prob_jf_2_s');
save('prob_jsR_2_s','prob_jsR_2_s');
save('prob_jstau_2_s','prob_jstau_2_s');

%% Figures
t = (1961.5:.25:2015.5)';

prob_M_regime_s = prob_M_regime_s;
prob_F_regime_s = prob_F_regime_s;
prob_I_regime_s = prob_I_regime_s;
prob_E_regime_s = prob_E_regime_s;

figure1 = figure;

% Create subplot
subplot1 = subplot(4,1,1,'Parent',figure1,'YTick',[0 0.5 1]);
xlim(subplot1,[1961.5 2015.5]);
ylim(subplot1,[0 1]);
box(subplot1,'on');
hold(subplot1,'all');

% Create plot
area(t,prob_M_regime_s,'Parent',subplot1,'LineWidth',2.5,'DisplayName','Prob(Regime M)','EdgeColor',[0 0 1],'FaceColor',[0 0 1]);

% Create subplot
subplot2 = subplot(4,1,2,'Parent',figure1,'YTick',[0 0.5 1]);
xlim(subplot2,[1961.5 2015.5]);
ylim(subplot2,[0 1]);
box(subplot2,'on');
hold(subplot2,'all');

% Create plot
area(t,prob_F_regime_s,'Parent',subplot2,'LineWidth',2.5,'DisplayName','Prob(Regime F)','EdgeColor',[0 1 0],'FaceColor', [0 1 0]);

% Create subplot
subplot3 = subplot(4,1,3,'Parent',figure1,'YTick',[0 0.5 1]);
xlim(subplot3,[1961.5 2015.5]);
ylim(subplot3,[0 1]);
box(subplot3,'on');
hold(subplot3,'all');

% Create plot
area(t,prob_I_regime_s,'Parent',subplot3,'LineWidth',2.5,'DisplayName','Prob(Regime I)','EdgeColor',[1 1 0],'FaceColor', [1 1 0]);

% Create subplot
subplot4 = subplot(4,1,4,'Parent',figure1,'YTick',[0 0.5 1]);
xlim(subplot4,[1961.5 2015.5]);
ylim(subplot4,[0 1]);
box(subplot4,'on');
hold(subplot4,'all');

% Create plot
area(t,prob_E_regime_s,'Parent',subplot4,'LineWidth',2.5,'DisplayName','Prob(Regime E)','EdgeColor',[1 0 0],'FaceColor', [1 0 0]);

% Create legend
legend1 = legend(subplot4,'show');
set(legend1,'Position',[0.617173291764093 0.197648376259798 0.112236944660951 0.0261291526689063],'EdgeColor',[1 1 1]);

% Create legend
legend2 = legend(subplot3,'show');
set(legend2,'Position',[0.617173291764093 0.426091825307951 0.112236944660951 0.0261291526689063],'EdgeColor',[1 1 1]);

% Create legend
legend3 = legend(subplot2,'show');
set(legend3,'Position',[0.617173291764093 0.66013437849944 0.116134060795012 0.0261291526689063],'EdgeColor',[1 1 1]);

% Create legend
legend4 = legend(subplot1,'show');
set(legend4,'Position',[0.246167835801506 0.882978723404255 0.118472330475448 0.0261291526689063],'EdgeColor',[1 1 1]);

%%

% Create figure
figure2 = figure;

% Create subplot
subplot21 = subplot(2,1,1,'Parent',figure2,'YTick',[0 0.5 1]);
xlim(subplot21,[1961.5 2015.5]);
ylim(subplot21,[0 1]);
box(subplot21,'on');
hold(subplot21,'all');

% Create plot
area(t,prob_jsR_2_s,'Parent',subplot21,'LineWidth',2.5,'DisplayName','Prob(High \sigma_R)',...
    'FaceColor',[0.501960813999176 0.501960813999176 0.501960813999176],...
    'EdgeColor',[0.501960813999176 0.501960813999176 0.501960813999176],...
    'LineWidth',2.5);
% Create subplot
subplot22 = subplot(2,1,2,'Parent',figure2,'YTick',[0 0.5 1]);
xlim(subplot22,[1961.5 2015.5]);
ylim(subplot22,[0 1]);
box(subplot22,'on');
hold(subplot22,'all');

% Create plot
area(t,prob_jstau_2_s,'Parent',subplot22,'LineWidth',2.5,'DisplayName','Prob(High \sigma_\tau)',...
    'FaceColor',[0.501960813999176 0.501960813999176 0.501960813999176],...
    'EdgeColor',[0.501960813999176 0.501960813999176 0.501960813999176],...
    'LineWidth',2.5);

% Create legend
legend21 = legend(subplot22,'show');
set(legend21,'Position',[0.578576010002594 0.345516427759803 0.106780982073266 0.0328480776409108],'EdgeColor',[1 1 1]);

% Create legend
legend22 = legend(subplot21,'show');
set(legend22,'Position',[0.578576010002594 0.822242654339003 0.106780982073266 0.0328480776409108],'EdgeColor',[1 1 1]);

%% Display Results
display('Parameter Estimates');
coefficients = {'R_c';
 'alphap_1';
 'alphap_2';
 'alphay_1';
 'alphay_2';
 'kappaRp_1';
 'kappaRp_2';
 'kappaRy_1';
 'kappaRy_2';
 'rhoR';
 'sigmaR_1';
 'sigmaR_2';
 'tau_c';
 'gammab_1';
 'gammab_2';
 'gammay_1';
 'gammay_2';
 'kappatauy_1';
 'kappatauy_2';
 'rhotau';
 'sigmatau_1';
 'sigmatau_2';
 'pmf_11';
 'pmf_21';
 'pmf_31';
 'pmf_12';
 'pmf_22';
 'pmf_32';
 'pmf_13';
 'pmf_23';
 'pmf_33';
 'pmf_14';
 'pmf_24';
 'pmf_34';
 'psR_11';
 'psR_12';
 'pstau_11';
 'pstau_12'};

dataset(coefficients,param_mean,tstatparam)

%% Forecasting Interest Rate
betaR_11 = [rhoR (1-rhoR)*[R_c alphap_1 alphay_1] sigmaR_1*kappaRp_1 sigmaR_1*kappaRy_1]';
betaR_12 = [rhoR (1-rhoR)*[R_c alphap_1 alphay_1] sigmaR_2*kappaRp_1 sigmaR_2*kappaRy_1]';
betaR_21 = [rhoR (1-rhoR)*[R_c alphap_2 alphay_2] sigmaR_1*kappaRp_2 sigmaR_1*kappaRy_2]';
betaR_22 = [rhoR (1-rhoR)*[R_c alphap_2 alphay_2] sigmaR_2*kappaRp_2 sigmaR_2*kappaRy_2]';
T = length(data);
R = data(2:T,1);
dzlb = (R==0.25); % Dummy ZLB
xR = [data(1:T-1,1) ones(T-1,1) data(2:T,3:4) data(2:T,6:7)];

prob_jm_2_s = smooth_prob(:,2)+smooth_prob(:,4) + smooth_prob(:,6)+smooth_prob(:,8) + smooth_prob(:,10)+smooth_prob(:,12) + smooth_prob(:,14)+smooth_prob(:,16);
prob_jsR_2_s = sum(smooth_prob(:,5:8),2)+sum(smooth_prob(:,13:16),2);

Rpredicted = (1-prob_jm_2_s).*(1-prob_jsR_2_s).*(xR*betaR_11) + (1-prob_jm_2_s).*prob_jsR_2_s.*(xR*betaR_12) + prob_jm_2_s.*(1-prob_jsR_2_s).*(xR*betaR_21) + prob_jm_2_s.*prob_jsR_2_s.*(xR*betaR_22);

figure3 = figure;
axes3 = axes('Parent',figure3,'FontSize',12);
xlim(axes3,[1961.5 2015.5]);
box(axes3,'on');
hold(axes3,'all');
plot31 = plot(t,R,'LineWidth',2.5,'LineWidth',2.5,'Color',[0 0 0]);
hold on
plot32 = plot(t,Rpredicted,'LineWidth',2.5,'LineStyle','--','Color',[0.313725501298904 0.313725501298904 0.313725501298904]);
plot3 = plot(t,zeros(size(t,1),1),'LineWidth',1,'LineStyle','--','Color',[0 0 0]);
legend31 = legend([plot31 plot32], 'R_t', 'R_t^* Predicted');
set(legend31,'Position',[0.640555988568459 0.657708100037327 0.116913484021824 0.0761478163493841],...
    'FontSize',14,...
    'FontName','Garamond', 'Box','off');
ylabel('%');
ylim([-2 20]);

%% Calculating Correlation Between Sm and Sf
A = [eye(4)-PSmf;ones(1,4)];
p_ = (A'*A)\A';
p = p_(:,5);
varm = (p(1)+p(3))*(1-p(1)-p(3));
varf = (p(1)+p(2))*(1-p(1)-p(2));
covmf = p(1)*p(4)-p(2)*p(3);
corrmf = covmf/sqrt(varm*varf);
